import memoize from 'micro-memoize';
import visit from 'unist-util-visit';
import pAll from 'p-all';
import lqip from 'lqip-modern';

var isRelative = /*#__PURE__*/memoize(function (node) {
  return node && node.properties && node.properties.src && node.properties.src.startsWith('/');
});
var getAspectRatio = /*#__PURE__*/memoize(function (dimensions) {
  if (!dimensions) return;
  var width = dimensions.width,
      height = dimensions.height;
  return height / width * 100;
});

var rehypeImageSize = function rehypeImageSize(options) {
  var visitor = function visitor(node) {
    try {
      var _temp2 = function () {
        if (isRelative(node)) {
          return Promise.resolve(lqip(pathPrefix + "/" + node.properties.src)).then(function (output) {
            var dimensions = {
              width: output.metadata.originalWidth,
              height: output.metadata.originalHeight
            };
            node.properties['dimensions'] = dimensions;
            node.properties['aspectRatio'] = getAspectRatio(dimensions);
            node.properties['preview'] = output.metadata.dataURIBase64;
          });
        }
      }();

      return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(function () {
        return node;
      }) : node);
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var transformer = function transformer(tree) {
    try {
      var nodes = [];
      visit(tree, 'element', function (node) {
        if (node.tagName !== 'img') {
          return;
        } else {
          nodes.push(node);
        }
      });
      return Promise.resolve(pAll(nodes.map(function (node) {
        return function () {
          return visitor(node);
        };
      }), {
        concurrency: 25
      })).then(function () {
        return tree;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var pathPrefix = (options === null || options === void 0 ? void 0 : options.pathPrefix) || 'public';
  return transformer;
};

module.exports = /*#__PURE__*/memoize(rehypeImageSize);
//# sourceMappingURL=rehype-lqip.esm.js.map
