import memoize from 'micro-memoize';
import visit from 'unist-util-visit';
import pAll from 'p-all';
// @ts-ignore
import lqip from 'lqip-modern';

interface RehypeImageOptions {
  pathPrefix?: string;
}

const isRelative = memoize(
  (node: any) =>
    node && node.properties && node.properties.src && node.properties.src.startsWith('/')
);

const getAspectRatio = memoize((dimensions?: { width: number; height: number }) => {
  if (!dimensions) return;

  const { width, height } = dimensions;

  return (height / width) * 100;
});

const rehypeImageSize = (options: RehypeImageOptions) => {
  const pathPrefix = options?.pathPrefix || 'public';
  async function transformer(tree: any) {
    const nodes: any = [];
    visit(tree, 'element', (node: any) => {
      if (node.tagName !== 'img') {
        return;
      } else {
        nodes.push(node);
      }
    });
    await pAll(
      nodes.map((node: any) => () => visitor(node)),
      { concurrency: 25 }
    );
    return tree;
  }
  async function visitor(node: any) {
    if (isRelative(node)) {
      const output = await lqip(`${pathPrefix}/${node.properties.src}`);
      const dimensions = {
        width: output.metadata.originalWidth,
        height: output.metadata.originalHeight,
      };
      node.properties['dimensions'] = dimensions;
      node.properties['aspectRatio'] = getAspectRatio(dimensions);
      node.properties['preview'] = output.metadata.dataURIBase64;
    }
    return node;
  }
  return transformer;
};

module.exports = memoize(rehypeImageSize);
